package sra.jun.goodies.animation;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Vector;
import sra.smalltalk.DependentEvent;
import sra.smalltalk.DependentListener;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StImage;
import sra.smalltalk.StMenu;
import sra.smalltalk.StMenuItem;
import sra.smalltalk.StMenuModel;
import sra.smalltalk.StRectangle;
import sra.smalltalk.StValueHolder;
import sra.smalltalk.StView;
import sra.jun.goodies.button.JunButtonModel;
import sra.jun.goodies.files.JunFileDialog;
import sra.jun.goodies.files.JunFileModel;
import sra.jun.goodies.movie.JunImagesToMovie;
import sra.jun.goodies.progress.JunProgress;
import sra.jun.goodies.track.JunTrackSliderModel;
import sra.jun.goodies.track.JunTrackSliderView;
import sra.jun.goodies.track.JunTrackerModel;
import sra.jun.system.framework.JunApplicationModel;
import sra.jun.system.support.JunSystem;

/**
 * JunCartoonMovie class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2004/02/05 (By Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun526 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCartoonMovie.java,v 6.14 2004/04/15 06:27:53 nisinaka Exp $
 */
public class JunCartoonMovie extends JunApplicationModel {

	protected JunTrackerModel trackerModel;
	protected JunTrackSliderModel trackerModel2;
	protected Vector imageCollection;
	protected Vector timeCollection;
	protected StValueHolder progressText;
	protected Thread playProcess;
	protected String displayName;
	protected StMenuModel menuModel;

	/**
	 * Initializing.
	 * 
	 * @category initialize-release
	 */
	public void initialize() {
		super.initialize();

		trackerModel = null;
		imageCollection = null;
		timeCollection = null;
		progressText = null;
		playProcess = null;
	}

	/**
	 * Answer total millisecond of current animation.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double totalMilliseconds() {
		double millisecondClockValue;
		millisecondClockValue = 0;
		for (int i = 0; i < this.timeCollection().size(); ++i) {
			double each = ((Double) this.timeCollection().get(i)).doubleValue();
			millisecondClockValue = millisecondClockValue + each;
		}
		return millisecondClockValue;
	}

	/**
	 * Answer now.
	 * 
	 * @return double
	 * @category playing
	 */
	public double now() {
		return this.trackerModel().doubleValue();
	}

	/**
	 * Answer current milliseconds.
	 * 
	 * @return double
	 * @category accessing
	 */
	public double currentMilliseconds() {
		return this.now() * this.totalMilliseconds();
	}

	/**
	 * Answer cumulative time until index.
	 * 
	 * @param anIndex int
	 * @return double
	 * @category accessing
	 */
	public double cumulativeTimeUntilIndex_(int anIndex) {
		if (anIndex < 0) {
			return 0;
		}
		if (anIndex >= this.timeCollection().size()) {
			return this.totalMilliseconds();
		}
		double millisecondClockValue = 0;
		for (int i = 0; i < anIndex + 1; ++i) {
			millisecondClockValue = millisecondClockValue + ((Double) this.timeCollection().get(i)).doubleValue();
		}
		return millisecondClockValue;
	}

	/**
	 * Answer cumulative time at a collection.
	 * 
	 * @param cumulativeTime double
	 * @return int
	 * @category accessing
	 */
	public int indexAtCumulativeTime_(double cumulativeTime) {
		int maxIndex = this.timeCollection().size();
		if (cumulativeTime <= 0 || cumulativeTime > this.totalMilliseconds()) {
			return 0;
		}
		for (int i = 0; i < maxIndex; ++i) {
			if (this.cumulativeTimeUntilIndex_(i - 1) <= cumulativeTime && cumulativeTime < this.cumulativeTimeUntilIndex_(i)) {
				return i;
			}
		}
		return maxIndex - 1;
	}

	/**
	 * Answer an image to display.
	 * 
	 * @return sra.smalltalk.StImage
	 * @category accessing
	 */
	public StImage imageToDisplay() {
		return this.imageAtCumulativeTime_(this.currentMilliseconds());
	}

	/**
	 * Answer an image at cumulative time.
	 * @param cumulativeTime double
	 * @return StImage
	 */
	public StImage imageAtCumulativeTime_(double cumulativeTime) {
		int imageIndex = this.indexAtCumulativeTime_(cumulativeTime);
		if (imageIndex < 0) {
			return this.defaultBlueImage();
		}
		return (StImage) this.imageCollection().get(imageIndex);
	}

	/**
	 * Answer display name.
	 * 
	 * @return java.lang.String
	 * @category accessing
	 */
	public String displayName() {
		if (displayName == null) {
			displayName = "";
		}
		return displayName;
	}

	/**
	 * Set display name.
	 * 
	 * @param aString java.lang.String
	 * @category accessing
	 */
	public void displayName_(String aString) {
		displayName = aString;
	}

	/**
	 * Update progress text.
	 * 
	 * @category updating
	 */
	public void updateProgressText() {
		this.progressText().value_(this.stringFromMilliseconds_((long) this.currentMilliseconds()) + " / " + this.stringFromMilliseconds_((long) this.totalMilliseconds()));
	}

	/**
	 * Answer progress text.
	 * 
	 * @return sra.smalltalk.StValueHolder
	 * @category accessing
	 */
	public StValueHolder progressText() {
		if (progressText == null) {
			progressText = new StValueHolder("");
			updateProgressText();
		}
		return progressText;
	}

	/**
	 * Answer time collection.
	 * 
	 * @return java.util.Vector
	 * @category accessing
	 */
	public java.util.Vector timeCollection() {
		if (timeCollection == null) {
			timeCollection = new Vector();
		}
		return timeCollection;
	}

	/**
	 * Answer image collection.
	 * 
	 * @return java.util.Vector
	 * @category accessing
	 */
	public java.util.Vector imageCollection() {
		if (imageCollection == null) {
			imageCollection = new Vector();
		}
		return imageCollection;
	}

	/**
	 * Answer a JunTrackerModel.
	 * If trackerModel is null, set trackerModel a new JunTrackerModel.
	 * 
	 * @return JunTrackerModel
	 * @category accessing
	 */
	public JunTrackerModel trackerModel() {
		if (trackerModel != null) {
			return trackerModel;
		}
		trackerModel = new JunTrackerModel();
		trackerModel.addDependent_(this);
		final JunCartoonMovie this_ = this;
		trackerModel.playButton().action_(new StBlockClosure() {
			public Object value_(Object anObject) {
				this_.playAction_((JunButtonModel) anObject);
				return null;
			}
		});
		trackerModel.loopButton().action_(new StBlockClosure() {
			public Object value_(Object anObject) {
				this_.loopAction_((JunButtonModel) anObject);
				return null;
			}
		});
		trackerModel.nextButton().action_(new StBlockClosure() {
			public Object value_(Object anObject) {
				this_.nextAction_((JunButtonModel) anObject);
				return null;
			}
		});
		trackerModel.previousButton().action_(new StBlockClosure() {
			public Object value_(Object anObject) {
				this_.previousAction_((JunButtonModel) anObject);
				return null;
			}
		});
		trackerModel.compute_(new StBlockClosure() {
			public Object value_(Object anObject) {
				this_.setTime_(((Double) anObject).doubleValue());
				return null;
			}
		});
		trackerModel.enableMarkers();
		return trackerModel;
	}

	/**
	 * Answer a JunTrackSliderModel.
	 * 
	 * @return JunTrackSliderModel
	 */
	public JunTrackSliderModel trackerModel2() {
		if (trackerModel2 == null) {
			trackerModel2 = new JunTrackSliderModel();
			trackerModel2.parentTracker_(this.trackerModel());
			final JunCartoonMovie this_ = this;
			trackerModel2.compute_(new StBlockClosure() {
				public Object value_(Object anObject) {
					this_.setTime2_(((Double) anObject).doubleValue());
					return null;
				}
			});
		}
		return trackerModel2;
	}

	/**
	 * Set play button action.
	 * 
	 * @param model JunButtonModel
	 * @category button actions
	 */
	public void playAction_(JunButtonModel model) {
		if (model.value()) {
			this.stop();
		} else {
			if (model._isPressedWithShiftDown()) {
				this.trackerModel().first();
			}
			this.start();
		}
		this.updatePlayMenuIndication();
	}

	/**
	 * Set loop button action.
	 * 
	 * @param model JunButtonModel
	 * @category button actions
	 */
	public void loopAction_(JunButtonModel model) {
		if (this.trackerModel().loopCondition().equals($("oneWay"))) {
			this.trackerModel().loop();
		} else {
			this.trackerModel().oneWay();
		}
		this.updatePlayMenuIndication();
	}

	/**
	 * Set next button action.
	 * 
	 * @param model JunButtonModel
	 * @category button actions
	 */
	public void nextAction_(JunButtonModel model) {
		if (model._isPressedWithShiftDown()) {
			this.last();
		} else {
			this.next();
		}
	}

	/**
	 * Set previous button action.
	 * 
	 * @param model JunButtonModel
	 * @category button actions
	 */
	public void previousAction_(JunButtonModel model) {
		if (model._isPressedWithShiftDown()) {
			this.first();
		} else {
			this.previous();
		}
	}

	/**
	 * Go to frame 0 (First of movie).
	 *
	 * @category playing
	 */
	public void prologue() {
		this.goto_(0);
	}

	/**
	 * Go to frame 1 (End of movie).
	 * 
	 * @category playing
	 */
	public void epilogue() {
		this.goto_(1);
	}

	/**
	 * Execute playing movie thread.
	 *
	 * @category playing
	 */
	public void run() {
		if (this.isEmpty()) {
			return;
		}
		if (this.now() == 1) {
			this.prologue();
		}
		if (this.trackerModel().isIntervalNotEmpty() && this.now() == this.trackerModel().lastMarker()) {
			this.goto_(this.trackerModel().firstMarker());
		}
		Thread.yield();
		try {
			Thread.sleep(this.tickTime());
		} catch (InterruptedException e) {
			System.err.println("Interrupted exception: e");
		}
		while (this.isPlay()) {
			double currentTime = this.currentMilliseconds() + this.tickTime();
			double endTime = this.totalMilliseconds();
			if (endTime <= 0) {
				this.epilogue();
			} else {
				this.goto_(currentTime / endTime);
			}
			if (currentTime >= endTime) {
				if (this.trackerModel().loopCondition() == $("loop")) {
					this.prologue();
				} else {
					this.stop();
				}
			}
			if (this.trackerModel().isIntervalNotEmpty()) {
				endTime = this.trackerModel().lastMarker() * this.totalMilliseconds();
				if (currentTime == endTime) {
					if (this.trackerModel().loopCondition() == $("loop")) {
						this.first();
					} else {
						this.stop();
					}
				}
			}
			Thread.yield();
			try {
				Thread.sleep(this.tickTime());
			} catch (InterruptedException e) {
				System.err.println("Interrupted exception: " + e);
			}
		}
	}

	/**
	 * Start to play a movie.
	 * 
	 * @category playing
	 */
	public void start() {
		if (this.isEmpty()) {
			return;
		}
		this.trackerModel().playButtonVisual_(true);
		final JunCartoonMovie this_ = this;
		playProcess = new Thread() {
			public void run() {
				try {
					Thread.yield();
					this_.run();
				} finally {
					Thread.yield();
					playProcess = null;
				}
			}
		};
		playProcess.setPriority(Thread.NORM_PRIORITY);
		playProcess.start();
		this.updatePlayMenuIndication();
	}

	/**
	 * Stop to playing movie.
	 * 
	 * @category playing
	 */
	public void stop() {
		if (this.isEmpty()) {
			return;
		}
		this.trackerModel().playButtonVisual_(false);
		this.updatePlayMenuIndication();
	}

	/**
	 * Go to last of movie.
	 * 
	 * @category playing
	 */
	public void last() {
		this.trackerModel().last();
	}

	/**
	 * Go to next frame of a movie file.
	 * 
	 * @category playing
	 */
	public void next() {
		if (this.isEmpty()) {
			return;
		}
		int currentIndex = this.indexAtCumulativeTime_(this.currentMilliseconds());
		double cumulativeTime = this.cumulativeTimeUntilIndex_(currentIndex);
		double totalValue = this.totalMilliseconds();
		double normalizedValue;
		if (totalValue <= 0) {
			normalizedValue = 1;
		} else {
			normalizedValue = cumulativeTime / totalValue;
		}
		this.goto_(normalizedValue);
	}

	/**
	 * Go to fisrt frame of a movie file.
	 * 
	 * @category playing
	 */
	public void first() {
		this.trackerModel().first();
	}

	/**
	 * Go to previous frame of a movie file.
	 * 
	 * @category playing
	 */
	public void previous() {
		if (this.isEmpty()) {
			return;
		}
		int currentIndex = this.indexAtCumulativeTime_(this.currentMilliseconds());
		double cumulativeTime = this.cumulativeTimeUntilIndex_(currentIndex);
		double keepTime = ((Double) this.timeCollection().get(currentIndex)).doubleValue();
		cumulativeTime = cumulativeTime - keepTime;
		if (this.currentMilliseconds() == cumulativeTime) {
			currentIndex = Math.max(currentIndex - 1, 0);
			keepTime = ((Double) this.timeCollection().get(currentIndex)).doubleValue();
			while (currentIndex > 1 && keepTime == 0) {
				currentIndex = Math.max(currentIndex - 1, 0);
				keepTime = ((Double) this.timeCollection().get(currentIndex)).doubleValue();
			}
			cumulativeTime = this.cumulativeTimeUntilIndex_(currentIndex);
			keepTime = ((Double) this.timeCollection().get(currentIndex)).doubleValue();
			cumulativeTime = cumulativeTime - keepTime;
		}
		double totalValue = this.totalMilliseconds();
		double normalizedValue;
		if (totalValue <= 0) {
			normalizedValue = 0;
		} else {
			normalizedValue = cumulativeTime / totalValue;
		}
		this.goto_(normalizedValue);
	}

	/**
	 * Go to normalizedValue time.
	 * 
	 * @param normalizedValue double
	 * @category playing
	 */
	public void goto_(double normalizedValue) {
		this.trackerModel().value_(normalizedValue);
	}

	/**
	 * Answer current animation is empty or not.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isEmpty() {
		return this.imageCollection().size() == 0;
	}

	/**
	 * Answer current animation is playing or not.
	 * 
	 * @return boolean
	 * @category testing
	 */
	public boolean isPlay() {
		return this.trackerModel().playButton().value() == true;
	}

	/**
	 * Answer default display name color.
	 * 
	 * @return Color
	 * @category defaults
	 */
	public Color defaultDisplayNameColor() {
		Color black = Color.black;
		Color green = Color.green;
		Color blendColor = new Color(black.getRed() + green.getRed(), black.getGreen() + green.getGreen(), black.getBlue() + green.getBlue());
		return blendColor;
	}

	/**
	 * Answer default movie player class.
	 * 
	 * @return Class
	 * @category defaults
	 */
	public Class defaultMoviePlayerClass() {
		return sra.jun.goodies.movie.JunMoviePlayer.class;
	}

	/**
	 * Answer default "blue image".
	 * 
	 * @return StImage
	 * @category defaults
	 */
	public StImage defaultBlueImage() {
		StImage aImage = new StImage(this.defaultBlueImageExtent());
		Graphics aGraphics = aImage.image().getGraphics();
		aGraphics.setColor(Color.blue);
		aGraphics.fillRect(0, 0, aImage.width(), aImage.height());
		return aImage;
	}

	/**
	 * Answer default blue image extent.
	 * 
	 * @return Point
	 * @category defaults
	 */
	public Point defaultBlueImageExtent() {
		return new Point(320, 240);
	}

	/**
	 * Set time.
	 * 
	 * @param time Time
	 * @category private
	 */
	public void setTime_(double time) {
		this.updateProgressText();
		this.changed_($("image"));
		this.setTime1_(time);
	}

	/**
	 * Set time.
	 * 
	 * @param  time Time
	 * @category private
	 */
	public void setTime1_(double time) {
		boolean aBoolean = this.trackerModel2().areMarkersActive();
		if (this.trackerModel().isIntervalNotEmpty()) {
			this.trackerModel2().enableMarkers();
		} else {
			this.trackerModel2().disableMarkers();
		}
		if (!(aBoolean == this.trackerModel2().areMarkersActive())) {
			DependentListener[] dependents = this.trackerModel2().dependents();
			for (int i = 0; i < dependents.length; ++i) {
				if (dependents[i] instanceof JunTrackSliderView) {
					dependents[i].update_(new DependentEvent(dependents[i], $("interval"), null));
				}
			}
		}
		double value;
		if (this.trackerModel().isIntervalNotEmpty()) {
			value = (this.trackerModel().doubleValue() - this.trackerModel().firstMarker()) / (this.trackerModel().lastMarker() - this.trackerModel().firstMarker());
			value = value * (1 - this.markerMargins()[0] - this.markerMargins()[1]) + this.markerMargins()[0];
			value = Math.max(0, Math.min(value, 1));
		} else {
			value = this.trackerModel().doubleValue();
		}
		double[] margins = { this.markerMargins()[0], 1 - this.markerMargins()[1] };
		this.trackerModel2().intervalHolder().value_(margins);
		this.trackerModel2().valueHolder().value_(value);
		DependentListener[] dependents = this.trackerModel2().dependents();
		for (int i = 0; i < dependents.length; ++i) {
			if (dependents[i] instanceof JunTrackSliderView) {
				dependents[i].update_(new DependentEvent(dependents[i], $("value"), null));
			}
		}
	}

	/**
	 * Set time.
	 * 
	 * @param time double
	 * @category private
	 */
	public void setTime2_(double time) {
		double value;
		if (this.trackerModel().isIntervalNotEmpty()) {
			value = (time - this.markerMargins()[0]) / (1 - this.markerMargins()[0] - this.markerMargins()[1]);
			value = (this.trackerModel().lastMarker() - this.trackerModel().firstMarker()) * value + this.trackerModel().firstMarker();
			this.trackerModel().value_(value);
		} else {
			this.trackerModel().value_(time);
		}
	}

	/**
	 * Answer marker margins.
	 * 
	 * @return double[]
	 * @category private
	 */
	public double[] markerMargins() {
		return new double[] { 0.1, 0.1 };
	}

	/**
	 * Answer tick time.
	 * 
	 * @return int
	 * @category private
	 */
	public int tickTime() {
		return 50;
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 * @category menu accessing
	 */
	public StMenuModel menuModel() {
		if (menuModel == null) {
			menuModel = defaultMenu();
		}

		return menuModel;
	}

	/**
	 * Create the menubar for JunCartoonMovie.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 * @category menu accessing
	 */
	protected StMenuModel defaultMenu() {
		menuModel = new StMenuModel();
		StMenu fileMenu = new StMenu(JunSystem.$String("File"));
		fileMenu.add(this.createMenuItem(JunSystem.$String("Save as movie..."), "saveAsMovie"));
		fileMenu.addSeparator();
		fileMenu.add(this.createMenuItem(JunSystem.$String("Quit"), "quitDoing"));
		menuModel.add(fileMenu);
		StMenu playMenu = new StMenu(JunSystem.$String("Play"));
		playMenu.add(this.createMenuItem(JunSystem.$String("Start"), "start"));
		playMenu.add(this.createMenuItem(JunSystem.$String("Stop"), "stop"));
		playMenu.addSeparator();
		playMenu.add(this.createMenuItem(JunSystem.$String("Loop"), "loopPlay"));
		playMenu.add(this.createMenuItem(JunSystem.$String("One way"), "oneWayPlay"));
		menuModel.add(playMenu);
		StMenu positionMenu = new StMenu(JunSystem.$String("Position"));
		positionMenu.add(this.createMenuItem(JunSystem.$String("Previous"), "gotoPrevious"));
		positionMenu.add(this.createMenuItem(JunSystem.$String("Next"), "gotoNext"));
		positionMenu.addSeparator();
		positionMenu.add(this.createMenuItem(JunSystem.$String("-> Prologue"), "gotoPrologue"));
		positionMenu.add(this.createMenuItem(JunSystem.$String("-> Epilogue"), "gotoEpilogue"));
		positionMenu.addSeparator();
		positionMenu.add(this.createMenuItem(JunSystem.$String("-> First"), "gotoFirst"));
		positionMenu.add(this.createMenuItem(JunSystem.$String("-> Last"), "gotoLast"));
		positionMenu.addSeparator();
		positionMenu.add(this.createMenuItem(JunSystem.$String("Set first marker"), "setFirstMarker"));
		positionMenu.add(this.createMenuItem(JunSystem.$String("Set last marker"), "setLastMarker"));
		menuModel.add(positionMenu);
		//		StMenu miscMenu = new StMenu("Misc");
		//		miscMenu.add(this.createMenuItem("Inspect", "inspectModel"));
		//		menuModel.add(miscMenu);

		return menuModel;
	}

	/**
	 * Update play menu indication.
	 * 
	 * @category menu accessing
	 */
	public void updatePlayMenuIndication() {
		StMenuItem menuItem = this.menuModel().getItem("start");
		if (menuItem != null) {
			if (this.isEmpty()) {
				menuItem.setEnabled(false);
			} else {
				if (this.isPlay()) {
					menuItem.setEnabled(false);
				} else {
					menuItem.setEnabled(true);
				}
			}
		}
		menuItem = this.menuModel().getItem("stop");
		if (menuItem != null) {
			if (this.isEmpty()) {
				menuItem.setEnabled(false);
			} else {
				if (this.isPlay()) {
					menuItem.setEnabled(true);
				} else {
					menuItem.setEnabled(false);
				}
			}
		}
		menuItem = this.menuModel().getItem("loopPlay");
		if (menuItem != null) {
			if (this.trackerModel().loopCondition().equals($("loop"))) {
				menuItem.setEnabled(false);
			} else {
				menuItem.setEnabled(true);
			}
		}
		menuItem = this.menuModel().getItem("oneWayPlay");
		if (menuItem != null) {
			if (this.trackerModel().loopCondition().equals($("oneWay"))) {
				menuItem.setEnabled(false);
			} else {
				menuItem.setEnabled(true);
			}
		}
	}

	/**
	 * Answer my default view.
	 * 
	 * @return StView
	 * @category interface opening
	 */
	public StView defaultView() {
		return this.defaultViewWithTracker();
	}

	/**
	 * Answer my default view with tracker.
	 * 
	 * @return sra.smalltalk.StView
	 * @category interface-opening
	 */
	public StView defaultViewWithTracker() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return JunCartoonMovieViewAwt.WithTracker(this);
		} else {
			return JunCartoonMovieViewSwing.WithTracker(this);
		}
	}

	/**
	 * Add an image and set keep time.
	 * 
	 * @param anImage StImage
	 * @param millisecondClockValue double
	 * @category adding
	 */
	public void addImage_keepTime_(StImage anImage, double millisecondClockValue) {
		this.imageCollection().add(anImage);
		this.timeCollection().add(new Double(millisecondClockValue));
		double totalValue = this.totalMilliseconds();
		if (totalValue <= 0) {
			this.trackerModel().step_(1);
		} else {
			this.trackerModel().step_((double) (this.tickTime() / totalValue));
		}
		this.updateMenuIndication();
	}

	/**
	 * Invoked when a window is in the process of being closed.
	 * 
	 * @param e java.awt.event.WindowEvent
	 * @see sra.smalltalk.StApplicationModel#noticeOfWindowClose()
	 * @category interface closing
	 */
	public void noticeOfWindowClose(WindowEvent e) {
		if (playProcess != null) {
			this.trackerModel().end();
			try {
				playProcess.join(1000);
			} catch (InterruptedException ie) {
			}
			playProcess = null;
		}

		super.noticeOfWindowClose(e);
	}

	/**
	 * Loop play menu message.
	 * 
	 * @category menu messages
	 */
	public void loopPlay() {
		this.trackerModel().loop();
		this.updatePlayMenuIndication();
	}

	/**
	 * OneWay play menu message.
	 * 
	 * @category menu messages
	 */
	public void oneWayPlay() {
		this.trackerModel().oneWay();
		this.updatePlayMenuIndication();
	}

	/**
	 * Go to epilogue menu message.
	 * 
	 * @category menu messages
	 */
	public void gotoEpilogue() {
		this.epilogue();
	}

	/**
	 * Go to first menu message.
	 * 
	 * @category menu messages
	 */
	public void gotoFirst() {
		this.first();
	}

	/**
	 * Go to last menu message.
	 * 
	 * @category menu messages
	 */
	public void gotoLast() {
		this.last();
	}

	/**
	 * Go to next menu message.
	 * 
	 * @category menu messages
	 */
	public void gotoNext() {
		this.next();
	}

	/**
	 * Go to previous menu message.
	 * 
	 * @category menu messages
	 */
	public void gotoPrevious() {
		this.previous();
	}

	/**
	 * Go to prologue menu message.
	 * 
	 * @category menu messages
	 */
	public void gotoPrologue() {
		this.prologue();
	}

	/**
	 * Set first marker menu message.
	 * 
	 * @category menu messages
	 */
	public void setFirstMarker() {
		this.trackerModel().firstMarkerButton().action().value_(this.trackerModel().firstMarkerButton());
	}

	/**
	 * Set last marker menu message.
	 * 
	 * @category menu messages
	 */
	public void setLastMarker() {
		this.trackerModel().lastMarkerButton().action().value_(this.trackerModel().lastMarkerButton());
	}

	/**
	 * Quit menu message.
	 * 
	 * @category menu messages
	 */
	public void quitDoing() {
		this.closeRequest();
	}

	/**
	 * Save as movie menu message.
	 * 
	 * @category menu messages
	 */
	public void saveAsMovie() {
		JunFileModel.FileType[] fileTypes = new JunFileModel.FileType[] { new JunFileModel.FileType(JunSystem.$String("<1p> files", null, JunSystem.$String("Movie")), new String[] { "*.mov", "*.MOV" })
		};
		File aFile = JunFileDialog.RequestNewFilename(JunSystem.$String("Input a <1p> file.", null, JunSystem.$String("Movie")), new File(this.defaultBaseName() + ".mov"), fileTypes, fileTypes[0]);
		if (aFile == null) {
			return;
		}

		this.saveAsMovieTo_(aFile);
		// (new JunMoviePlayer(aFile)).open();
	}

	/**
	 * Save as movie to aFilename menu message.
	 * 
	 * @param aFilename java.lang.String
	 * @category menu messages
	 */
	public void saveAsMovieTo_(File aFilename) {
		final StView aView = this.getView();
		StRectangle aBox;
		if (aView == null) {
			aBox = new StRectangle(0, 0, 320, 240);
		} else {
			aBox = new StRectangle(aView.toComponent().getBounds());
		}
		final JunCartoonMovie this_ = this;
		(new JunImagesToMovie(aFilename.getPath(), new Dimension(aBox.width(), aBox.height()))).do_(new StBlockClosure() {
			public Object value_(Object o) {
				final JunImagesToMovie imagesToMovie = (JunImagesToMovie) o;
				if (aView != null) {
					imagesToMovie.background_(aView.toComponent().getBackground());
				}
				(new JunProgress()).do_(new StBlockClosure() {
					public Object value_(Object o) {
						final JunProgress aProgress = (JunProgress) o;
						final Integer[] timeAndCount = { new Integer(0), new Integer(0)};
						final int size = this_.imageCollection().size();
						aProgress.value_(timeAndCount[1].intValue() / size);
						aProgress.message_(JunSystem.$String("writing..."));
						this_.imagesAndTimesDo_(new StBlockClosure() {
							public Object value_value_(Object A, Object B) {
								StImage anImage = (StImage) A;
								int aTick = ((Double) B).intValue();
								if (aTick > 0) {
									imagesToMovie.add_milliseconds_(anImage, aTick);
								}
								timeAndCount[1] = new Integer(timeAndCount[1].intValue() + 1);
								timeAndCount[0] = new Integer(timeAndCount[0].intValue() + aTick);
								aProgress.value_(timeAndCount[1].intValue() / size);
								aProgress.message_(JunSystem.$String("writing..."));
								return null;
							}
						});
						return Boolean.TRUE;
					}
				});
				return null;
			}
		});
	}

	/**
	 * Do images and times.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @category enumerating
	 */
	public void imagesAndTimesDo_(StBlockClosure aBlock) {
		for (int i = 0; i < this.imageCollection().size(); ++i) {
			StImage anImage = (StImage) this.imageCollection().get(i);
			Double millisecondClockValue = (Double) this.timeCollection().get(i);
			aBlock.value_value_(anImage, millisecondClockValue);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Cartoon Movie Player");
	}

	/**
	 * Answer my KeyListener which handles my keyboard events.
	 *
	 * @return java.awt.event.KeyListener
	 * @category keyboard
	 */
	public KeyListener _keyListener() {
		return new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if (_keyboardEvent(e)) {
					e.consume();
				}
			}
		};
	}

	/**
	 * Process the key event.
	 *
	 * @param ev java.awt.event.KeyEvent
	 * @return boolean
	 * @category keyboard
	 */
	protected boolean _keyboardEvent(KeyEvent ev) {
		if (ev.isConsumed()) {
			return false;
		}

		int code = ev.getKeyCode();
		switch (code) {
			case KeyEvent.VK_SPACE :
			case KeyEvent.VK_ENTER :
				if (this.isPlay()) {
					this.stop();
				} else {
					this.start();
				}
				return true;
			case KeyEvent.VK_LEFT :
				this.previousAction_(this.trackerModel().previousButton());
				return true;
			case KeyEvent.VK_RIGHT :
				this.nextAction_(this.trackerModel().nextButton());
				return true;
			case KeyEvent.VK_TAB :
				if (this.trackerModel().loopCondition() == $("oneWay")) {
					this.trackerModel().loopCondition_($("loop"));
				} else {
					this.trackerModel().loopCondition_($("oneWay"));
				}
				return true;
			case (int) '[' :
				this.trackerModel().firstMarker_(this.now());
				return true;
			case (int) ']' :
				this.trackerModel().lastMarker_(this.now());
				return true;
			case (int) 'P' :
				this.prologue();
				return true;
			case (int) 'E' :
				this.epilogue();
				return true;
			case (int) 'F' :
				this.trackerModel().first();
				return true;
			case (int) 'L' :
				this.trackerModel().last();
				return true;
		}
		return false;
	}

	/**
	 * Answer a view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category accessing
	 */
	public StView getView() {
		Object[] dependents = this.dependents();
		for (int i = 0; i < dependents.length; i++) {
			Object each = dependents[i];
			if (each instanceof JunCartoonMovieView && ((StView) each).model() == this) {
				return (StView) each;
			}
		}
		return null;
	}

	/**
	 * Convert the number in milliseconds to a string representation.
	 *
	 * @param milliseconds long
	 * @return java.lang.String
	 * @category private
	 */
	protected String stringFromMilliseconds_(long milliseconds) {
		long decimal = milliseconds % 1000;
		long value = milliseconds / 1000;
		int seconds = (int) (value % 60);
		int minutes = (int) (value / 60 % 60);
		int hours = (int) (value / 3600);

		StringBuffer buf = new StringBuffer();
		if (hours < 10) {
			buf.append('0');
		}
		buf.append(hours);
		buf.append(':');
		if (minutes < 10) {
			buf.append('0');
		}
		buf.append(minutes);
		buf.append(':');
		if (seconds < 10) {
			buf.append('0');
		}
		buf.append(seconds);
		buf.append('.');
		if (decimal < 100) {
			buf.append('0');
		}
		if (decimal < 10) {
			buf.append('0');
		}
		buf.append(decimal);
		return buf.toString();
	}

}
