package sra.jun.goodies.bulletinboard;

import sra.smalltalk.*;

/**
 * JunBulletinBoardController class
 * 
 *  @author    nisinaka
 *  @created   2002/11/27 (by nisinaka)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on JunXXX for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunBulletinBoardController.java,v 6.5 2004/04/15 06:27:54 nisinaka Exp $
 */
public class JunBulletinBoardController extends StController {

	protected boolean bulletinState;
	protected Thread bulletinProcess;

	/**
	 * Create a new instance of JunBulletinBoardController and initialize it.
	 */
	public JunBulletinBoardController() {
		bulletinState = false;
		bulletinProcess = null;
	}

	/**
	 * Make sure to terminate the process when finalizing.
	 */
	protected void finalize() throws Throwable {
		this.terminate();
		super.finalize();
	}

	/**
	 * Resure the process.
	 */
	protected void resume() {
		bulletinState = true;
		bulletinProcess = new Thread() {
			public void run() {
				try {
					while (bulletinState && view() != null && model().dependents().length > 0) {
						((JunBulletinBoardView) view()).display();
						Thread.yield();
						try {
							Thread.sleep(tickTime());
						} catch (InterruptedException e) {
							bulletinState = false;
						}
					}
				} finally {
					terminate();
				}
			}
		};
		// bulletinProcess.setPriority(Thread.NORM_PRIORITY + 1);
		bulletinProcess.setPriority(Thread.MAX_PRIORITY);
		bulletinProcess.start();
	}

	/**
	 * Terminate the process.
	 */
	protected void terminate() {
		bulletinState = false;
		Thread.yield();
	}

	/**
	 * Answer the time for ticking the bulletin process.
	 *
	 * @return int
	 */
	protected int tickTime() {
		return 20;
	}
}
