package sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

import sra.jun.system.support.JunSystem;
import sra.smalltalk.StSimpleDialogViewSwing;
import sra.smalltalk.WindowSpecLayout;

/**
 * JunCalendarDateAndTimeIntervalDialogViewSwing class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun472 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarDateAndTimeIntervalDialogViewSwing.java,v 6.8 2004/04/15 06:27:55 nisinaka Exp $
 */
public class JunCalendarDateAndTimeIntervalDialogViewSwing
	extends StSimpleDialogViewSwing
	implements JunCalendarDateAndTimeIntervalDialogView {
	protected JButton acceptButton;
	protected JButton cancelButton;
	protected JunCalendarDateViewSwing fromDateView;
	protected JunCalendarDateViewSwing toDateView;
	protected JunCalendarTimeViewSwing fromTimeView;
	protected JunCalendarTimeViewSwing toTimeView;

	/**
	 * Create a new instance of JunCaledarDateAndTimeIntervalDialogViewSwing and initialize it with the JunCalendarDateAndTimeIntervalDialog.
	 * @param newCalendarDateAndDateAndTimeIntervalDialog JunCalendarDateAndTimeIntervalDialog
	 * @category instance creation
	 */
	public JunCalendarDateAndTimeIntervalDialogViewSwing(JunCalendarDateAndTimeIntervalDialog newCalendarDateAndTimeIntervalDialog) {
		super(newCalendarDateAndTimeIntervalDialog);
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 * @category defaults
	 */
	public Dialog defaultDialog() {
		Dialog aDialog =
			new Dialog(
				new Frame(),
				this.getCalendarDateAndTimeIntervalDialog().windowTitle(),
				true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as a JunCalendarDateAndTimeIntervalDialog.
	 * @return JunCalendarDateAndTimeIntervalDialog
	 * @category accessing
	 */
	public JunCalendarDateAndTimeIntervalDialog getCalendarDateAndTimeIntervalDialog() {
		return (JunCalendarDateAndTimeIntervalDialog) this.model();
	}

	/**
	 * Answer the panel on accept/cancel buttons.
	 *
	 * @return java.awt.Panel
	 * @category actins
	 */
	protected Panel _acceptCancelButtons() {
		Panel panel = new Panel();
		panel.setLayout(new FlowLayout());
		panel.add(this.acceptButton());
		panel.add(this.cancelButton());
		return panel;
	}

	/**
	 * Answer my accept button.
	 * @return javax.swing.jButton
	 * @category actions
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton(JunSystem.$String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateAndTimeIntervalDialog()
						.acceptButtonPressed())
						.booleanValue()) {
						getCalendarDateAndTimeIntervalDialog().accept().value_(
							true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.fromDateView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 0f, 300, 0.5f, 0));
		this.add(this.fromTimeView(), WindowSpecLayout.Position(0f, 300, 0f, 1, 1f, -102, 0.5f, 0));
		this.add(this.toDateView(), WindowSpecLayout.Position(0f, 1, 0.5f, 0, 0f, 300, 1f, -1));
		this.add(this.toTimeView(), WindowSpecLayout.Position(0f, 300, 0.5f, 0, 1f, -102, 1f, -1));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -96, 0f, 56, 1f, -8, 0f, 88));
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -96, 0f, 16, 1f, -8, 0f, 48));
		defaultButton = this.acceptButton();
		this.setPreferredSize(new Dimension(550, 400));
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return javax.swing.jButton
	 * @category actions
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton(JunSystem.$String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarDateAndTimeIntervalDialog()
						.cancelButtonPressed())
						.booleanValue()) {
						getCalendarDateAndTimeIntervalDialog().cancel().value_(
							true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer from date view.
	 *
	 * @return JunCalendarDateViewSwing
	 * @category accessing
	 */
	protected JunCalendarDateViewSwing fromDateView() {
		if (fromDateView == null) {
			fromDateView =
				new JunCalendarDateViewSwing(
					this
						.getCalendarDateAndTimeIntervalDialog()
						.calendarDateAndTimeIntervalModel()
						.fromCalendarDateAndTimeModel()
						.calendarDateModel());
		}
		return fromDateView;
	}

	/**
	 * Answer to date view.
	 *
	 * @return JunCalendarDateViewSwing
	 * @category accessing
	 */
	protected JunCalendarDateViewSwing toDateView() {
		if (toDateView == null) {
			toDateView =
				new JunCalendarDateViewSwing(
					this
						.getCalendarDateAndTimeIntervalDialog()
						.calendarDateAndTimeIntervalModel()
						.toCalendarDateAndTimeModel()
						.calendarDateModel());
		}
		return toDateView;
	}

	/**
	 * Answer from Time view.
	 *
	 * @return JunCalendarTimeViewSwing
	 * @category accessing
	 */
	protected JunCalendarTimeViewSwing fromTimeView() {
		if (fromTimeView == null) {
			fromTimeView =
				new JunCalendarTimeViewSwing(
					this
						.getCalendarDateAndTimeIntervalDialog()
						.calendarDateAndTimeIntervalModel()
						.fromCalendarDateAndTimeModel()
						.calendarTimeModel());
		}
		return fromTimeView;
	}

	/**
	 * Answer to Time view.
	 *
	 * @return JunCalendarTimeViewSwing
	 * @category accessing
	 */
	protected JunCalendarTimeViewSwing toTimeView() {
		if (toTimeView == null) {
			toTimeView =
				new JunCalendarTimeViewSwing(
					this
						.getCalendarDateAndTimeIntervalDialog()
						.calendarDateAndTimeIntervalModel()
						.toCalendarDateAndTimeModel()
						.calendarTimeModel());
		}
		return toTimeView;
	}
}
