package sra.jun.goodies.calendar;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;

import sra.jun.system.support.JunSystem;
import sra.smalltalk.StSimpleDialogViewSwing;
import sra.smalltalk.WindowSpecLayout;

/**
 * JunCalendarTimeDialogViewSwing class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/06 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun472 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunCalendarTimeDialogViewSwing.java,v 6.8 2004/04/15 06:27:55 nisinaka Exp $
 */
public class JunCalendarTimeDialogViewSwing
	extends StSimpleDialogViewSwing
	implements JunCalendarTimeDialogView {
	protected JunCalendarTimeViewSwing calendarTimeView;
	protected JButton acceptButton;
	protected JButton cancelButton;

	/**
	 * Create a new instance of JunCaledarTimeDialogViewSwing and initialize it with the JunCalendarTimeDialog.
	 * @param newCalendarTimeAndTimeDialog JunCalendarTimeDialog
	 * @category instance creation
	 */
	public JunCalendarTimeDialogViewSwing(JunCalendarTimeDialog newCalendarTimeDialog) {
		super(newCalendarTimeDialog);
	}

	/**
	 * Answer the default dialog.
	 *
	 * @return java.awt.Dialog
	 * @category accessing
	 */
	public Dialog defaultDialog() {
		Dialog aDialog =
			new Dialog(
				new Frame(),
				this.getCalendarTimeDialog().windowTitle(),
				true);
		aDialog.add(this);
		aDialog.pack();
		return aDialog;
	}

	/**
	 * Answer my model as a JunCalendarTimeDialog.
	 * @return JunCalendarTimeDialog
	 * @category accessing
	 */
	public JunCalendarTimeDialog getCalendarTimeDialog() {
		return (JunCalendarTimeDialog) this.model();
	}

	/**
	 * Answer my accept button.
	 * @return javax.swing.jButton
	 * @category accessing
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton(JunSystem.$String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarTimeDialog()
						.acceptButtonPressed())
						.booleanValue()) {
						getCalendarTimeDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 * @category interface opening
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.calendarTimeView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -102, 1f, -1));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -96, 0f, 56, 1f, -8, 0f, 88));
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -96, 0f, 16, 1f, -8, 0f, 48));
		defaultButton = this.acceptButton();
		this.setPreferredSize(new Dimension(250, 200));
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return javax.swing.jButton
	 * @category accessing
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton(JunSystem.$String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (((Boolean) getCalendarTimeDialog()
						.cancelButtonPressed())
						.booleanValue()) {
						getCalendarTimeDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer my calendar date view.
	 *
	 * @return JunCalendarTimeViewSwing
	 * @category accessing
	 */
	protected JunCalendarTimeViewSwing calendarTimeView() {
		if (calendarTimeView == null) {
			calendarTimeView =
				new JunCalendarTimeViewSwing(
					this.getCalendarTimeDialog().calendarTimeModel());
		}
		return calendarTimeView;
	}
}
