package sra.jun.goodies.colors;

import java.awt.*;
import java.awt.event.*;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.jun.goodies.cursors.JunCursors;
import sra.jun.system.framework.JunDialog;
import sra.jun.system.support.JunSystem;
import sra.smalltalk.*;

/**
 * JunColorChoiceController class
 * 
 *  @author    Hirotsugu Kondo
 *  @created   1998/11/05 (by Hirotsugu Kondo)
 *  @updated   1999/08/31 (by Mitsuhiro Asada)
 *  @updated   2003/04/02 (by Mituhiro Asada)
 *  @version   445 (with StPL6.5) based on Jun490 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunColorChoiceController.java,v 6.8 2004/04/15 06:27:55 nisinaka Exp $
 */
public class JunColorChoiceController extends StController implements MouseListener, MouseMotionListener {
	/**
	 * Invoked when the mouse has been clicked on a component.
	 * 
	 * @param event java.awt.event.MouseEvent
	 * @category events
	 */
	public void mouseClicked(MouseEvent event) {
		if (event.isMetaDown() || event.isPopupTrigger()) {
			super.mouseClicked(event);
			return;
		}

		Point point = event.getPoint();
		JunColorChoiceView view = this.getView();
		if (view.spuitArea().contains(point)) {
			Color color = JunColorSpuit.Request();
			if (event.isShiftDown() == false) {
				this.getModel().color_(color);
			}
		}
	}

	/**
	 * Invoked when the mouse button has been moved.
	 * 
	 * @param event java.awt.event.MouseEvent
	 * @category events
	 */
	public void mouseDragged(MouseEvent event) {
		Point point = event.getPoint();
		JunColorChoiceView view = this.getView();
		if ((view.plateArea().contains(point)) || (view.barArea().contains(point))) {
			this.mousePressed(event);
		}
	}

	/**
	 * Tempolary cursor set method. Not called now.
	 * 
	 * @param event java.awt.event.MouseEvent
	 * @category events
	 */
	public void mouseMoved(MouseEvent event) {
		Point point = event.getPoint();
		JunColorChoiceView view = this.getView();
		if ((view.plateArea().contains(point)) || (view.barArea().contains(point)) || (view.hueArea().contains(point)) || (view.saturationArea().contains(point)) || (view.brightnessArea().contains(point)) || (view.colorArea().contains(point))) {
			((Component) view).setCursor(JunCursors.CrossCursor());
		} else {
			((Component) view).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}

	/**
	 * Invoked when a mouse button has been pressed. press button to set
	 * color.
	 * 
	 * @param event java.awt.event.MouseEvent
	 * @category events
	 */
	public void mousePressed(MouseEvent event) {
		if (event.isMetaDown() || event.isPopupTrigger()) {
			super.mousePressed(event);
			return;
		}

		Point point = event.getPoint();
		JunColorChoiceView view = this.getView();
		Point dialogPoint = view.topComponent().getLocation();
		dialogPoint.translate(point.x, point.y);

		if (view.plateArea().contains(point)) {
			double x = (double) (point.x - view.plateArea().getLocation().x) / (view.plateArea().width - 1.0);
			double y = (double) (point.y - view.plateArea().getLocation().y) / (view.plateArea().height - 1.0);
			Jun2dPoint jPoint = new Jun2dPoint(x, y);
			this.getModel().xy_(jPoint);
		}

		if (view.barArea().contains(point)) {
			double z = (double) (point.y - view.barArea().getLocation().y) / (view.barArea().height - 1.0);
			this.getModel().z_(z);
		}

		if (view.hueArea().contains(point)) {
			String result = JunDialog.Request_(JunSystem.$String("hue?"), view.hueString());
			if (result != null) {
				try {
					double resultValue = new Double(result).doubleValue();
					if ((resultValue >= 0.0) && (resultValue <= 1.0)) {
						this.getModel().hue_(resultValue);
					}
				} catch (NumberFormatException e) {
				}
			}
		}

		if (view.saturationArea().contains(point)) {
			String result = JunDialog.Request_(JunSystem.$String("saturation?"), view.saturationString());
			if (result != null) {
				try {
					double resultValue = new Double(result).doubleValue();
					if ((resultValue >= 0.0) && (resultValue <= 1.0)) {
						this.getModel().saturation_(resultValue);
					}
				} catch (NumberFormatException e) {
				}
			}
		}

		if (view.brightnessArea().contains(point)) {
			String result = JunDialog.Request_(JunSystem.$String("brightness?"), view.brightnessString());
			if (result != null) {
				try {
					double resultValue = new Double(result).doubleValue();
					if ((resultValue >= 0.0) && (resultValue <= 1.0)) {
						this.getModel().brightness_(resultValue);
					}
				} catch (NumberFormatException e) {
				}
			}
		}

		if (view.colorArea().contains(point)) {
			System.out.println(view.colorString());
		}
	}

	/**
	 * Add listeners.
	 * 
	 * @param newView sra.mvctest.StView
	 * @category view accessing
	 */
	protected void buildListener(StView newView) {
		Component aView = (Component) newView;
		aView.addMouseListener(this);
		aView.addMouseMotionListener(this);
	}

	/**
	 * Answer this model as a JunColorChoiceModel.
	 * 
	 * @return sra.jun.goodies.colors.JunColorChoiceModel
	 * @category model accessing
	 */
	protected JunColorChoiceModel getModel() {
		return (JunColorChoiceModel) this.model();
	}

	/**
	 * Answer this view as a JunColorChoiceView.
	 * 
	 * @return sra.jun.goodies.colors.JunColorChoiceView
	 * @category view accessing
	 */
	protected JunColorChoiceView getView() {
		return (JunColorChoiceView) this.view();
	}
}