package sra.jun.goodies.files;


import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import sra.smalltalk.StSimpleDialogViewSwing;
import sra.smalltalk.WindowSpecLayout;
import sra.jun.system.support.JunSystem;

/**
 * JunFileDialogViewSwing class
 * 
 *  @author    nisinaka
 *  @created   2003/03/12 (by nisinaka)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun433 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFileDialogViewSwing.java,v 6.6 2004/04/15 06:27:56 nisinaka Exp $
 */
public class JunFileDialogViewSwing extends StSimpleDialogViewSwing implements JunFileDialogView {

	protected JunFileViewSwing fileView;
	protected JButton acceptButton;
	protected JButton cancelButton;

	/**
	 * Create a new instance of JunFileDialogViewSwing and initialize it.
	 *
	 * @param aFileDialog sra.jun.goodies.files.JunFileDialog
	 */
	public JunFileDialogViewSwing(JunFileDialog aFileDialog) {
		super(aFileDialog);
	}

	/**
	 * Answer my model as JunFileDialog.
	 *
	 * @return sra.jun.goodies.files.JunFileDialog
	 */
	public JunFileDialog getFileDialog() {
		return (JunFileDialog) this.model();
	}

	/**
	 * Answer my accpet button.
	 *
	 * @return javax.swing.JButton
	 */
	protected JButton acceptButton() {
		if (acceptButton == null) {
			acceptButton = new JButton(JunSystem.$String("Accept"));
			acceptButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (getFileDialog().acceptButtonPressed()) {
						getFileDialog().accept().value_(true);
					}
				}
			});
		}
		return acceptButton;
	}

	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setLayout(new WindowSpecLayout());
		this.add(this.fileView(), WindowSpecLayout.Position(0f, 1, 0f, 1, 1f, -97, 1f, -1));
		this.add(this.acceptButton(), WindowSpecLayout.Position(1f, -96, 0f, 16, 1f, -8, 0f, 48));
		this.add(this.cancelButton(), WindowSpecLayout.Position(1f, -96, 0f, 56, 1f, -8, 0f, 88));
		this.setPreferredSize(new Dimension(540, 300));

		defaultButton = this.acceptButton();
	}

	/**
	 * Answer my cancel button.
	 *
	 * @return javax.swing.JButton
	 */
	protected JButton cancelButton() {
		if (cancelButton == null) {
			cancelButton = new JButton(JunSystem.$String("Cancel"));
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (getFileDialog().cancelButtonPressed()) {
						getFileDialog().cancel().value_(true);
					}
				}
			});
		}
		return cancelButton;
	}

	/**
	 * Answer my file view.
	 *
	 * @return sra.jun.goodies.files.JunFileViewSwing
	 */
	protected JunFileViewSwing fileView() {
		if (fileView == null) {
			fileView = new JunFileViewSwing(this.getFileDialog().fileModel());
		}
		return fileView;
	}
}
