package sra.jun.goodies.freehand;

import java.util.Vector;
import sra.jun.goodies.lisp.JunLispCons;
import sra.jun.goodies.lisp.JunLispList;
import sra.jun.system.support.JunSystem;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StMenu;
import sra.smalltalk.StMenuModel;

/**
 * JunFreehandNote class
 * 
 *  @author    nisinaka
 *  @created   2002/07/09 (by nisinaka)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun380 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunFreehandNote.java,v 6.9 2004/04/15 06:27:57 nisinaka Exp $
 */
public class JunFreehandNote extends JunFreehandPage {

	protected int pageIndex;
	protected Vector pageCollection;
	protected StMenuModel _menuModel;

	/**
	 * Create an instance of JunFreehandNote and initialize it.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunFreehandNote() {
		super();
	}

	/**
	 * Create an instance of JunFreehandNote and initialize it with the JunLispList.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @category Instance creation
	 */
	public JunFreehandNote(JunLispList aList) {
		this();
		this.fromLispList_(aList);
	}

	/**
	 * Initialize the instance variables.
	 * 
	 * @category initialize-release
	 */
	protected void initialize() {
		super.initialize();

		this.flushNote();
		_menuModel = null;
	}

	/**
	 * Flush the note.
	 * 
	 * @category accessing
	 */
	public void flushNote() {
		pageIndex = -1;
		pageCollection = new Vector();
		this.makePage();
	}

	/**
	 * Answer the current collection of the pages.
	 * 
	 * @return java.util.Vector
	 * @category accessing
	 */
	public Vector pageCollection() {
		return pageCollection;
	}

	/**
	 * Answer the current index of the pages.
	 * 
	 * @return int
	 * @category accessing
	 */
	public int pageIndex() {
		return pageIndex;
	}

	/**
	 * Answer a menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 * @category menu accessing
	 */
	public StMenuModel menuModel() {
		if (_menuModel == null) {
			_menuModel = this.defaultMenu();
		}

		return _menuModel;
	}

	/**
	 * Answer the default menu model.
	 * 
	 * @return sra.smalltalk.StMenuModel
	 * @category menu accessing
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();

		StMenu fileMenu = new StMenu(JunSystem.$String("File"));
		fileMenu.add(createMenuItem(JunSystem.$String("New"), "newNote"));
		fileMenu.addSeparator();
		fileMenu.add(createMenuItem(JunSystem.$String("Quit"), "closeRequest"));
		rootMenu.add(fileMenu);

		StMenu editMenu = new StMenu(JunSystem.$String("Edit"));
		editMenu.add(createMenuItem(JunSystem.$String("Back"), "goBackward"));
		editMenu.add(createMenuItem(JunSystem.$String("Forward"), "goForward"));
		editMenu.addSeparator();
		editMenu.add(createMenuItem(JunSystem.$String("Nib"), "changeNibWidthAndColor"));
		editMenu.add(createMenuItem(JunSystem.$String("Clear"), "clearPage"));
		rootMenu.add(editMenu);

		StMenu pageMenu = new StMenu(JunSystem.$String("Page"));
		pageMenu.add(createMenuItem(JunSystem.$String("Next (New Page)"), "goToNextPage"));
		pageMenu.add(createMenuItem(JunSystem.$String("Previous"), "goToPreviousPage"));
		pageMenu.addSeparator();
		pageMenu.add(createMenuItem(JunSystem.$String("First"), "goToFirstPage"));
		pageMenu.add(createMenuItem(JunSystem.$String("Last"), "goToLastPage"));
		rootMenu.add(pageMenu);

		StMenu miscMenu = new StMenu(JunSystem.$String("Misc"));
		miscMenu.add(createMenuItem(JunSystem.$String("Viewfinder"), "openViewfinder"));
		rootMenu.add(miscMenu);

		return rootMenu;
	}

	/**
	 * Menu message: go to the first page.
	 * 
	 * @category menu messages
	 */
	public void goToFirstPage() {
		this.savePage();
		pageIndex = 0;
		this.loadPage();
		this.changed();
	}

	/**
	 * Menu message: go to the last page.
	 * 
	 * @category menu messages
	 */
	public void goToLastPage() {
		this.savePage();
		pageIndex = this.pageCollection().size() - 1;
		this.loadPage();
		this.changed();
	}

	/**
	 * Menu message: go to the next page.
	 * 
	 * @category menu messages
	 */
	public void goToNextPage() {
		this.savePage();

		if (this.pageIndex() < (this.pageCollection().size() - 1)) {
			pageIndex++;
		} else if (!this.isEmpty()) {
			this.makePage();
		}

		this.loadPage();
		this.changed();
	}

	/**
	 * Menu message: go to the previous page.
	 * 
	 * @category menu messages
	 */
	public void goToPreviousPage() {
		this.savePage();

		if (this.pageIndex() <= 0) {
			return;
		}

		pageIndex--;
		this.loadPage();
		this.changed();
	}

	/**
	 * Menu message: create a new note.
	 * 
	 * @category menu messages
	 */
	public void newNote() {
		(new JunFreehandNote()).open();
	}

	/**
	 * Answer the title string.
	 * 
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Freehand Note");
	}

	/**
	 * Load the current page.
	 * 
	 * @category private
	 */
	protected void loadPage() {
		JunFreehandPage aPage = (JunFreehandPage) this.pageCollection().get(this.pageIndex());
		this.nibColor_nibWidth_nibCollection_locusCollection_scalarCollection_currentIndex_(aPage.nibColor(), aPage.nibWidth(), aPage.nibCollection(), aPage.locusCollection(), aPage.scalarCollection(), aPage.currentIndex());
	}

	/**
	 * Make a page.
	 * 
	 * @category private
	 */
	protected void makePage() {
		this.pageCollection().add(new JunFreehandPage());
		pageIndex++;
	}

	/**
	 * Save the current page.
	 * 
	 * @category private
	 */
	protected void savePage() {
		JunFreehandPage aPage = new JunFreehandPage();
		aPage.nibColor_nibWidth_nibCollection_locusCollection_scalarCollection_currentIndex_(this.nibColor(), this.nibWidth(), this.nibCollection(), this.locusCollection(), this.scalarCollection(), this.currentIndex());
		this.pageCollection().set(this.pageIndex(), aPage);
	}

	/**
	 * Get my attributes from the lisp list.
	 * 
	 * @param aList aList sra.jun.goodies.lisp.JunLispList
	 * @see sra.jun.goodies.freehand.JunFreehandPage#fromLispList(sra.jun.goodies.lisp.JunLispList)
	 * @category lisp support
	 */
	public void fromLispList_(JunLispList aList) {
		super.fromLispList_(aList);
		this.pageIndexFromLispList_(aList);
		this.pageCollectionFromLispList_(aList);
	}

	/**
	 * Get my page index from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void pageIndexFromLispList_(JunLispList aList) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object car) {
				return new Boolean(car instanceof JunLispCons && ((JunLispCons) car).head() == $("pageIndex"));
			}
		}, new StBlockClosure());
		if (list == null) {
			return;
		}

		pageIndex = ((Number) list.tail()).intValue();
	}

	/**
	 * Get my page collection from the lisp list.
	 * 
	 * @param aList sra.jun.goodies.lisp.JunLispList
	 * @category lisp support
	 */
	protected void pageCollectionFromLispList_(JunLispList aList) {
		JunLispCons list = (JunLispCons) aList.detect_ifNone_(new StBlockClosure() {
			public Object value_(Object car) {
				return new Boolean(car instanceof JunLispCons && ((JunLispCons) car).head() == $("pageCollection"));
			}
		}, new StBlockClosure());
		if (list == null) {
			return;
		}

		Object[] anArray = ((JunLispCons) list.tail()).asArray();
		Vector aVector = new Vector(anArray.length);
		for (int i = 0; i < anArray.length; i++) {
			aVector.add(new JunFreehandPage((JunLispCons) anArray[i]));
		}

		pageCollection = aVector;
	}

	/**
	 * Convert the receiver to a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 * @see sra.jun.goodies.freehand.JunFreehandPage#toLispList()
	 * @category lisp support
	 */
	public JunLispCons toLispList() {
		JunLispCons list = super.toLispList();
		list.add_(this.pageIndexToLispList());
		list.add_(this.pageCollectionToLispList());
		return list;
	}

	/**
	 * Convert the receiver's page index to a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons pageIndexToLispList() {
		JunLispCons list = DefaultLispCons();
		list.head_($("pageIndex"));
		list.tail_(new Integer(this.pageIndex()));
		return list;
	}

	/**
	 * Convert the receiver's nib collection to a lisp list.
	 * 
	 * @return sra.jun.goodies.lisp.JunLispCons
	 * @category lisp support
	 */
	protected JunLispCons pageCollectionToLispList() {
		JunLispCons list = DefaultLispCons();
		list.head_($("pageCollection"));
		JunLispList[] pageList = new JunLispList[this.pageCollection().size()];
		for (int i = 0; i < pageList.length; i++) {
			pageList[i] = ((JunFreehandPage) this.pageCollection().get(i)).toLispList();
		}
		list.tail_(JunLispList.List_(pageList));
		return list;
	}

}
