package sra.jun.goodies.nib;

import java.awt.Dialog;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StView;
import sra.jun.system.support.JunSystem;

/**
 * JunNibChoiceWithColorDialog class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2004/03/26 (By Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun490 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunNibChoiceWithColorDialog.java,v 6.2 2004/04/15 06:28:00 nisinaka Exp $
 */
public class JunNibChoiceWithColorDialog extends JunNibChoiceDialog {

	protected JunNibChoiceWithColor nibChoiceWithColorModel;

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request() {
		return Request(JunSystem.$String("Nib Choice Dialog"));
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @return java.lang.Object
	 * @return Object
	 * @category Utilities
	 */
	public static Object Request(String messageString) {
		return Request(messageString, new JunNibChoiceWithColor());
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param nibChoiceModel sra.jun.goodies.nib.JunNibChoiceWithColor
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request(String messageString, JunNibChoiceWithColor aNibChoiceModel) {
		return Request(messageString, aNibChoiceModel, null, null);
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param nibChoiceModel sra.jun.goodies.nib.JunNibChoice
	 * @param acceptBlock sra.smalltalk.StBlockClosure
	 * @param cancelBlock sra.smalltalk.StBlockClosure
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request(String messageString, JunNibChoiceWithColor aNibChoiceModel, StBlockClosure acceptBlock, StBlockClosure cancelBlock) {
		JunNibChoiceWithColorDialog nibChoiceDialog = new JunNibChoiceWithColorDialog();
		if (aNibChoiceModel != null) {
			nibChoiceDialog.nibChoiceWithColorModel_(aNibChoiceModel);
		}
		Dialog dialog = nibChoiceDialog.allButOpenDialog();
		if (messageString != null) {
			dialog.setTitle(messageString);
		}
		if (acceptBlock != null) {
			nibChoiceDialog.acceptBlock_(acceptBlock);
		}
		if (cancelBlock != null) {
			nibChoiceDialog.cancelBlock_(cancelBlock);
		}
		_ShowAtMousePoint(dialog);
		if (nibChoiceDialog.accept()._booleanValue()) {
			return nibChoiceDialog.nibChoiceWithColorModel().value();
		} else {
			return null;
		}
	}

	/**
	 * Open a nib choice dialog and let a user to choose.
	 * 
	 * @param messageString java.lang.String
	 * @param valueArray java.lang.Object[]
	 * @return java.lang.Object
	 * @category Utilities
	 */
	public static Object Request(String messageString, Object[] valueArray) {
		return JunNibChoiceWithColorDialog.Request(messageString, new JunNibChoiceWithColor(valueArray), null, null);
	}

	/**
	 * Answer the current nib choice with color model.
	 * 
	 * @return sra.jun.goodies.JunNibChoiceWithColor
	 * @category accessing
	 */
	public JunNibChoiceWithColor nibChoiceWithColorModel() {
		if (nibChoiceWithColorModel == null) {
			nibChoiceWithColorModel = new JunNibChoiceWithColor();
		}
		return nibChoiceWithColorModel;
	}

	/**
	 * Set new nib choice with color model.
	 * 
	 * @param aNibChoiceModel sra.jun.goodies.nib.JunNibChoiceWithColor
	 * @category accessing
	 */
	public void nibChoiceWithColorModel_(JunNibChoiceWithColor aNibChoiceModel) {
		nibChoiceWithColorModel = aNibChoiceModel;
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunNibChoiceWithColorDialogViewAwt(this);
		} else {
			return new JunNibChoiceWithColorDialogViewSwing(this);
		}
	}

	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Nib Choice Dialog");
	}
}
