package sra.jun.goodies.plotter;

import java.awt.Color;
import java.awt.Frame;
import java.awt.List;
import java.lang.reflect.Method;
import java.util.ListIterator;
import java.util.Vector;
import sra.smalltalk.StBlockClosure;
import sra.smalltalk.StComposedText;
import sra.smalltalk.StMenu;
import sra.smalltalk.StMenuItem;
import sra.smalltalk.StSymbol;
import sra.smalltalk.StView;
import sra.jun.geometry.basic.Jun2dPoint;
import sra.jun.goodies.pen.JunPen;
import sra.jun.goodies.pen.JunPenDirection;
import sra.jun.goodies.pen.JunPenLocation;
import sra.jun.system.framework.JunApplicationModel;
import sra.jun.system.support.JunSystem;

/**
 * JunPlotter class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/30 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on JunXXX for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPlotter.java,v 1.8 2004/04/15 06:28:01 nisinaka Exp $
 */
public class JunPlotter extends JunApplicationModel {
	private Vector drawings;
	protected StMenu popupMenu; // i.e. yellowButtonMenu
	
	/**
	 * Create a new JunPlotter object.
	 *
	 * @category instance creation
	 */
	public JunPlotter() {
		super();
		this.initialize();
	}
	
	/**
	 *  Initialize JunPlotter object.
	 * 
	 * @category initialize-release
	 */
	public void initialize() {
		super.initialize();
		drawings = new Vector();
	}
	
	/**
	 * Drawing aBlock.
	 * @param sra.smalltalk.StBlockClosure
	 * 
	 * @category accessing
	 */
	public void drawingsDo_(StBlockClosure aBlock) {
		for (ListIterator array = drawings.listIterator(); array.hasNext();) {
			aBlock.value_(array.next());
		}
	}
	
	/**
	 * Answer methods of category programs.
	 * 
	 * @return java.lang.String[]
	 * @category accessing
	 */
	public String[] programSelectors() {
		String[] selectors = {
			"curveOfSinAndCos1",
			"curveOfSinAndCos2",
			"penExample",
			"sample1",
			"sample2",
			"sample3",
			"sample4",
			"sample5",
			"sample6",
			"sample7",
			"sample8",
			"sample9"
		};
		return selectors;
	}
	
	/**
	 * Browse programs.
	 * 
	 * @category command messages
	 */
	public void browsePrograms() {
		Method[] methods = this.getClass().getDeclaredMethods();
		Vector programs = new Vector();
		for (int i = 0; i < methods.length; ++i) {
			for (int j = 0; j < this.programSelectors().length; ++j) {
				if (methods[i].getName().equals(this.programSelectors()[j])) {
					programs.add(methods[i].getName());
				}
			}
		}
		List aList = new List(programs.size(), true);
		int width = 0, height = 0;
		StComposedText composedText = null;
		for (int i = 0; i < programs.size(); ++i) {
			aList.add((String)(programs.get(i)));
			composedText = new StComposedText((String)(programs.get(i)));
			if (width < composedText.width()) {
				width = composedText.width();
			}
			height += composedText.height();
		}
		Frame listWindow = new Frame();
		listWindow.add(aList);
		listWindow.setSize(width, height);
		listWindow.show();
	}
	
	/**
	 * Create a new JunPen.
	 * 
	 * @return sra.jun.goodies.pen.JunPen
	 * @category command messages
	 */
	public JunPen newPen() {
		JunPen aPen = new JunPen();
		aPen.compute_(new StBlockClosure() {
				public Object value_(Object array) {
					drawings.add(array);
					this.changed_with_($("draw"), array);
					return (Object)this;
				}
		});
		return aPen;
	}
	
	/**
	 * Clear drawing view.
	 * 
	 * @category command messages
	 */
	public void clearPaper() {
		drawings = new Vector();
		this.changed_($("clear"));
	}

	/**
	 * Answer my default view.
	 * 
	 * @return sra.smalltalk.StView
	 * @category interface opening
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunPlotterViewAwt(this);
		} else {
			return new JunPlotterViewSwing(this);
		}
	}
	
	/**
	 * Example: Curve of sin and cos 1.
	 *
	 * @category programs
	 */
	public void curveOfSinAndCos1() {
		JunPen pen1 = this.newPen();
		pen1.color_(Color.red);
		JunPen pen2 = this.newPen();
		pen2.color_(Color.blue);
		final double pi = Math.PI;
		final Jun2dPoint scale = new Jun2dPoint(30.0, 30.0);
		StBlockClosure curve = new StBlockClosure() {
			public Object value_value_(Object pen, Object function) {
				JunPen aPen = (JunPen)pen;
				StSymbol aFunction = (StSymbol)function;
				double x = 0, y = 0;
				aPen.up();
				for (double n = -3.0; n <= 3.0; n = n + 0.06) {
					x = n * pi;
					if (aFunction == $("sin")) {
						y = Math.sin(n * pi);
					}
					if (aFunction == $("cos")) {
						y = Math.cos(n * pi);
					}
					Jun2dPoint p = (new Jun2dPoint(x, y)).scaledBy_(scale);
					aPen.goto_(p);
					if (aPen.isUp()) {
						aPen.down();
					}
				}
				return null;
			}
		};
		curve.value_value_(pen1, $("sin"));
		curve.value_value_(pen2, $("cos"));
		this.changed_($("visual"));
	}
	
	/**
	 * Example: Curve of sin and cos 2.
	 *
	 * @category programs
	 */
	public void curveOfSinAndCos2() {
		this.clearPaper();
		JunPen pen0 = this.newPen();
		pen0.color_(Color.gray);
		JunPen pen1 = this.newPen();
		pen1.color_(Color.red);
		JunPen pen2 = this.newPen();
		pen2.color_(Color.blue);
		final double pi = Math.PI;
		final Jun2dPoint scale = new Jun2dPoint(30.0, 30.0);
		StBlockClosure line = new StBlockClosure() {
			public Object value_value_value_(Object pen, Object p1, Object p2) {
				JunPen pen_ = (JunPen)pen;
				Jun2dPoint p1_ = (Jun2dPoint)p1;
				Jun2dPoint p2_ = (Jun2dPoint)p2;
				pen_.location_(p1_.scaledBy_(scale));
				pen_.goto_(p2_.scaledBy_(scale));
				return (Object)this;
			}
		};
		StBlockClosure curve = new StBlockClosure() {
			public Object value_value_(Object pen, Object function) {
				JunPen aPen = (JunPen)pen;
				StSymbol aFunction = (StSymbol)function;
				double x = 0, y = 0;
				aPen.up();
				for (double n = -3; n <= 3; n = n + 0.06) {
					x = n * pi;
					if (aFunction == $("sin")) {
						y = Math.sin(n * pi);
					}
					if (aFunction == $("cos")) {
						y = Math.cos(n * pi);
					}
					Jun2dPoint p = (new Jun2dPoint(x, y)).scaledBy_(scale);
					aPen.goto_(p);
					if (aPen.isUp()) {
						aPen.down();
					}
				}
				return null;
			}
		};
		line.value_value_value_(pen0, new Jun2dPoint(-3 * pi, 0), new Jun2dPoint(3 * pi, 0));
		line.value_value_value_(pen0, new Jun2dPoint(0, -1), new Jun2dPoint(0, 1));
		curve.value_value_(pen1, $("sin"));
		curve.value_value_(pen2, $("cos"));
		this.changed_($("visual"));
	}
	
	/**
	 * Example: penExample.
	 * 
	 * @category programs
	 */
	public void penExample() {
		this.clearPaper();
		JunPen aPen = this.newPen();
		aPen.nib_(4);
		aPen.color_(Color.gray);
		for (int i = 1; i <= 50; ++i) {
			aPen.go_(i * 4);
			aPen.turn_(89);
		}
	}
	
	/**
	 * Example: Sample1.
	 * 
	 * @category programs
	 */
	public void sample1() {
		this.clearPaper();
		JunPen aPen = this.newPen();
		aPen.color_(Color.red);
		for (int i = 0; i <= 360; i = i + 45) {
			for (int j = 0; j < 3; ++j) {
				aPen.go_(100);
				aPen.turn_(120);
			}
			aPen.turn_(i);
		}
	}
	
	/**
	 * Example: Sample2.
	 * 
	 * @category programs
	 */
	public void sample2() {
		this.clearPaper();
		JunPen aPen = this.newPen();
		aPen.nib_(3);
		aPen.color_(Color.red);
		aPen.spiral_angle_(200, 144);
	}
	
	/**
	 * Example: Sample3.
	 * 
	 * @category programs
	 */
	public void sample3() {
		this.clearPaper();
		JunPen aPen = this.newPen();
		aPen.nib_(3);
		aPen.color_(Color.red);
		aPen.dragon_distance_(9, 5);
	}
	
	/**
	 * Example: Sample4.
	 * 
	 * @category programs
	 */
	public void sample4() {
		this.clearPaper();
		JunPen aPen = this.newPen();
		aPen.nib_(3);
		aPen.color_(Color.red);
		aPen.mandala_diameter_(24, 250);
	}
	
	/**
	 * Example: Sample5.
	 * 
	 * @category programs
	 */
	public void sample5() {
		this.clearPaper();
		final JunPlotter this_ = this;
		StBlockClosure creator = new StBlockClosure() {
			public Object value() {
				JunPen pen = this_.newPen();
				pen.color_(Color.red);
				return (Object)pen;
			}
		};
		final StBlockClosure spiral = new StBlockClosure() {
			public Object value_(Object pen) {
				JunPen aPen = (JunPen)pen;
				aPen.spiral_angle_(100, 88);
				return (Object)this;
			}
		};
		final JunPen pen1 = (JunPen)creator.value();
		final JunPen pen2 = (JunPen)creator.value();
		final JunPen pen3 = (JunPen)creator.value();
		final JunPen pen4 = (JunPen)creator.value();
		final JunPen pen5 = (JunPen)creator.value();
		pen1.location_(new Jun2dPoint(-75.0, -75.0));
		pen2.location_(new Jun2dPoint(75.0, -75.0));
		pen3.location_(new Jun2dPoint(75.0, 75.0));
		pen4.location_(new Jun2dPoint(-75.0, 75.0));
		pen5.location_(new Jun2dPoint(0.0, 0.0));
		new StBlockClosure() {
			public Object value() {
				spiral.value_(pen1);
				return null;
			}
		}.value();
		new StBlockClosure() {
			public Object value() {
				spiral.value_(pen2);
				return null;
			}
		}.value();
		new StBlockClosure() {
			public Object value() {
				spiral.value_(pen3);
				return null;
			}
		}.value();
		new StBlockClosure() {
			public Object value() {
				spiral.value_(pen4);
				return null;
			}
		}.value();
		new StBlockClosure() {
			public Object value() {
				spiral.value_(pen5);
				return null;
			}
		}.value();
	}
	
	/**
	 * Example: Sample6.
	 * 
	 * @category programs
	 */
	public void sample6() {
		this.clearPaper();
		final JunPlotter this_ = this;
		StBlockClosure creator = new StBlockClosure() {
			public Object value() {
				JunPen pen = this_.newPen();
				pen.color_(Color.red);
				return (Object)pen;
			}
		};
		final StBlockClosure spiral = new StBlockClosure() {
			public Object value_(Object pen) {
				JunPen aPen = (JunPen)pen;
				aPen.spiral_angle_(100, 88);
				return (Object)this;
			}
		};
		final JunPen pen1 = (JunPen)creator.value();
		final JunPen pen2 = (JunPen)creator.value();
		final JunPen pen3 = (JunPen)creator.value();
		final JunPen pen4 = (JunPen)creator.value();
		final JunPen pen5 = (JunPen)creator.value();
		pen1.location_(new Jun2dPoint(-75.0, -75.0));
		pen2.location_(new Jun2dPoint(75.0, -75.0));
		pen3.location_(new Jun2dPoint(75.0, 75.0));
		pen4.location_(new Jun2dPoint(-75.0, 75.0));
		pen5.location_(new Jun2dPoint(0.0, 0.0));
		Thread t1 = new Thread() {
			public void run() {
				spiral.value_(pen1);
			}
		};
		Thread t2 = new Thread() {
			public void run() {
				spiral.value_(pen2);
			}
		};
		Thread t3 = new Thread() {
			public void run() {
				spiral.value_(pen3);
			}
		};
		Thread t4 = new Thread() {
			public void run() {
				spiral.value_(pen4);
			}
		};
		Thread t5 = new Thread() {
			public void run() {
				spiral.value_(pen5);
			}
		};
		t1.start();
		t2.start();
		t3.start();
		t4.start();
		t5.start();
	}
	
	/**
	 * Example: Sample7.
	 * 
	 * @category programs
	 */
	public void sample7() {
		this.clearPaper();
		JunPen aPen = this.newPen();
		aPen.nib_(3);
		double[] location = {-50.0, -50.0, -50.0};
		aPen.location_(JunPenLocation.FromArray_(location));
		aPen.direction_(JunPenDirection.Zero());
		aPen.color_(Color.blue);
		for (int i = 0; i < 4; ++i) {
			aPen.go_(100);
			aPen.turn_(90);
		}
		location = new double[3];
		location[0] = 50.0; location[1] = -50.0; location[2] = -50.0;
		aPen.location_(JunPenLocation.FromArray_(location));
		aPen.direction_(JunPenDirection.Longitude_(90));
		aPen.color_(Color.blue);
		for (int i = 0; i < 4; ++i) {
			aPen.go_(100);
			aPen.tilt_(90);
		}
		location = new double[3];
		location[0] = -50.0; location[1] = 50.0; location[2] = -50.0;
		aPen.location_(JunPenLocation.FromArray_(location));
		aPen.direction_(JunPenDirection.Zero());
		aPen.color_(Color.blue);
		for (int i = 0; i < 4; ++i) {
			aPen.go_(100);
			aPen.tilt_(90);
		}
	}
	
	/**
	 * Example: Sample8.
	 * 
	 * @category programs
	 */
	public void sample8() {
		this.clearPaper();
		final JunPen aPen = this.newPen();
		aPen.nib_(2);
		double[] location = {-50.0, -50.0, -50.0};
		aPen.location_(JunPenLocation.FromArray_(location));
		aPen.direction_(JunPenDirection.Zero());
		aPen.color_(Color.red);
		aPen.fill_(new StBlockClosure() {
			public Object value() {
				for (int i = 0; i < 4; ++i) {
					aPen.go_(100);
					aPen.turn_(90);
				}
				return (Object)this;
			}
		});
		location[0] = 50.0;
		aPen.location_(JunPenLocation.FromArray_(location));
		aPen.direction_(JunPenDirection.Longitude_(90));
		aPen.color_(Color.green);
		aPen.fill_(new StBlockClosure() {
			public Object value() {
				for (int i = 0; i < 4; ++i) {
					aPen.go_(100);
					aPen.tilt_(90);
				}
				return (Object)this;
			}
		});
		location [0] = -50.0;
		location [1] = 50.0;
		aPen.location_(JunPenLocation.FromArray_(location));
		aPen.direction_(JunPenDirection.Zero());
		aPen.color_(Color.blue);
		aPen.fill_(new StBlockClosure() {
			public Object value() {
				for (int i = 0; i < 4; ++i) {
					aPen.go_(100);
					aPen.tilt_(90);
				}
				return (Object)this;
			}
		});
	}
	
	/**
	 * Example: Sample9.
	 * 
	 * @category programs
	 */
	public void sample9() {
		this.clearPaper();
		final JunPen aPen = this.newPen();
		aPen.nib_(1);
		aPen.color_(Color.blue);
		StBlockClosure aBlock = new StBlockClosure() {
			public Object value_(Object aSign) {
				int sign = ((Integer)aSign).intValue();
				aPen.home();
				for (int i = 1; i <= 50; ++i) {
					JunPenLocation from = aPen.location();
					aPen.up();
					aPen.go_(i * 4 * sign);
					aPen.goto_(aPen.location().plus_((JunPenLocation.X_y_z_(0.0, 2.0 * sign, 0.0))));
					JunPenLocation to = aPen.location();
					aPen.location_(from);
					aPen.down();
					aPen.goto_(to);
					aPen.tilt_(89);
				}
				return (Object)this;
			}
		};
		aBlock.value_((Object)(new Integer(1)));
		aBlock.value_((Object)(new Integer(-1)));
	}
	
	/**
	 * Flush menus.
	 * 
	 * @category flusing
	 */
	public void flushMenus() {
		popupMenu = null;
	}
	
	/**
	* Answer the popup menu if exist.
	*
	* @return sra.smalltalk.StMenu
	* @category menu accessing
	*/
   protected StMenu popupMenu() {
		String[] selectors = this.programSelectors();
		StMenuItem item = null;
		if (popupMenu != null) {
			item = popupMenu.getItem(JunSystem.$String("Execute") + "...");
			if (popupMenu != null) {
   				
			}
		}
		if (popupMenu == null || item != null) {
			popupMenu = new StMenu("");
			popupMenu.add(this.createMenuItem(JunSystem.$String("Browse Programs", "Browse"), "browsePrograms"));
			popupMenu.add(this.createMenuItem(JunSystem.$String("Clear"), "clearPaper"));
			popupMenu.add(this.popupSubMenu());
			popupMenu.add(this.createMenuItem(JunSystem.$String("Zoom in"), "zoomIn"));
			popupMenu.add(this.createMenuItem(JunSystem.$String("Zoom out"), "zoomOut"));
			popupMenu.add(this.createMenuItem(JunSystem.$String("Reset"), "resetView"));
		}
	   return popupMenu;
   }
   
   /**
    * Reset view.
    * 
    * @category command messages
    */
   public void resetView() {
   		this.plotterController().resetView();
   }
   
   /**
	* Zoom out.
	* 
	* @category command messages
	*/
   public void zoomOut() {
		this.plotterController().zoomOut();
   }
   
   /**
	* Zoom in.
	* 
	* @category command messages
	*/
   public void zoomIn() {
		this.plotterController().zoomIn();
   }
   
   /**
    * Answer sub menu.
    * 
    * @return sra.smalltalk.StMenu
    * @category menu accessing
    */
   public StMenu popupSubMenu() {
   		String[] selectors = this.programSelectors();
   		if (selectors == null) {
   			return null;
   		}
   		StMenu subMenu = new StMenu(JunSystem.$String("Execute") + "...");
   		for (int i = 0; i < selectors.length; ++i) {
   			subMenu.add(this.createMenuItem(selectors[i], selectors[i]));
   		}
   		return subMenu;
   }
	
   /**
	* Answer a plotter controller object.
	* 
	* @return JunPlotterController
	* @category controller accessing
	*/
   public JunPlotterController plotterController() {
	   return (JunPlotterController)this.getController();
   }

	/**
	 * Answer the window title
	 * 
	 * @return java.lang.String
	 * @see sra.smalltalk.StApplicationModel#windowTitle()
	 * @category interface opening
	 */
	protected String windowTitle() {
		return JunSystem.$String("Pen Plotter");
	}

}