package sra.jun.goodies.plotter;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import sra.jun.geometry.basic.Jun2dPoint;
import sra.jun.goodies.pen.JunPenTransformation;
import sra.smalltalk.StController;
import sra.smalltalk.StView;

/**
 * JunPlotterController class
 * 
 *  @author    Nobuto Matsubara
 *  @created   2003/10/30 (by Nobuto Matsubara)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun473 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunPlotterController.java,v 1.6 2004/04/15 06:28:01 nisinaka Exp $
 */
public class JunPlotterController extends StController implements MouseListener, MouseMotionListener {
	private MouseEvent mEvent;
	private Point _previousPoint;
	private Point _lastDownPoint;

	/**
	 * general dragging action.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseDragged(MouseEvent event) {
		mEvent = event;
		if (!mEvent.isMetaDown()) {
			this.scrollView();
		}
	}
	
	/**
	 * Invoked when a mouse button has been pressed on a component.
	 * 
	 * @param e java.awt.event.MouseEvent
	 * @category control defaults
	 */
	public void mousePressed(MouseEvent event) {
		_previousPoint = _lastDownPoint;
		_lastDownPoint = event.getPoint();
		if (event.isMetaDown()) {
			// yellow button activity
			this.view()._showPopupMenu(event.getX(), event.getY());
		} else {
			((Component) view).setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		}
	}

	/**
	 * Invoked when a mouse button has been released. cursor reset to default.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseReleased(MouseEvent event) {
		((Component) view).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
	
	/**
	 * Reset view.
	 * 
	 * @category menu messages
	 */
	public void resetView() {
		this.plotterView().viewingTransformation_(null);
	}
	
	/**
	 * Scroll view.
	 * 
	 * @category menu messages
	 */
	public void scrollView() {
		JunPenTransformation aTransformation = JunPenTransformation.Unity();
		aTransformation = aTransformation.product_(this.plotterView().viewingTransformation());
		Point currentPoint = mEvent.getPoint();
		if (_previousPoint != currentPoint) {
			JunPenTransformation aT = JunPenTransformation.Translate_(new Jun2dPoint(currentPoint.x - _previousPoint.x, currentPoint.y - _previousPoint.y));
			aTransformation = aTransformation.product_(aT);
			this.plotterView().viewingTransformation_(aTransformation);
			_previousPoint = currentPoint;
		}
	}
	
	/**
	 * Zoom in.
	 * 
	 *  @category menu messages
	 */
	public void zoomIn() {
		JunPenTransformation aTransformation = JunPenTransformation.Unity();
		aTransformation = aTransformation.product_(this.plotterView().viewingTransformation());
		aTransformation = aTransformation.product_(JunPenTransformation.Scale_(new Double(2.0)));
		this.plotterView().viewingTransformation_(aTransformation);
	}
	
	/**
	 * Zoom out.
	 * 
	 *  @category menu messages
	 */
	public void zoomOut() {
		JunPenTransformation aTransformation = JunPenTransformation.Unity();
		aTransformation = aTransformation.product_(this.plotterView().viewingTransformation());
		aTransformation = aTransformation.product_(JunPenTransformation.Scale_(new Double(0.5)));
		this.plotterView().viewingTransformation_(aTransformation);
	}
	
	/**
	 * Answer plotter view object.
	 * 
	 * @return sra.jun.goodies.plotter.JunPlotterView
	 * @category view accessing
	 */
	public JunPlotterView plotterView() {
		return (JunPlotterView)this.view();
	}

	/**
	 * add listeners.
	 * 
	 * @param newView sra.mvctest.StView
	 */
	protected void buildListener(StView newView) {
		final Component aView = (Component) newView;
		aView.addMouseListener(this);
		aView.addMouseMotionListener(this);
	}
}
