package sra.jun.goodies.sound;

import java.util.Hashtable;
import sra.smalltalk.*;
import sra.jun.system.framework.JunApplicationModel;

/**
 * JunSoundMeterModel class
 * 
 *  @author    Hoshi Takanori
 *  @created   2003/08/29 (by Hoshi Takanori)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun433 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunSoundMeterModel.java,v 6.5 2004/04/15 06:28:03 nisinaka Exp $
 */
public class JunSoundMeterModel extends JunApplicationModel {

	protected StValueHolder soundValueHolder;
	protected Hashtable soundLevelData;

	/**
	 * Answer the sound level data.
	 *
	 * @return java.util.Hashtable
	 */
	public Hashtable soundLevelData() {
		if (soundLevelData == null) {
			soundLevelData = new Hashtable();
		}
		return soundLevelData;
	}

	/**
	 * Answer the sound level data at key.
	 *
	 * @param key java.lang.Object
	 * @return double[]
	 */
	public double[] soundLevelDataAt_(Object key) {
		if (this.soundLevelData().containsKey(key)) {
			Object value = this.soundLevelData().get(key);
			if (value instanceof double[] && ((double[]) value).length >= 2) {
				return (double[]) value;
			}
		}
		return null;
	}

	/**
	 * Answer the sound value holder.
	 *
	 * @return sra.smalltalk.StValueHolder
	 */
	public StValueHolder soundValueHolder() {
		return soundValueHolder;
	}

	/**
	 * Set the sound value holder to valueHolder.
	 *
	 * @param valueHolder sra.smalltalk.StValueHolder
	 */
	public void soundValueHolder_(StValueHolder valueHolder) {
		if (soundValueHolder != null) {
			soundValueHolder.removeDependent_(this);
		}
		soundValueHolder = valueHolder;
		if (soundValueHolder != null) {
			soundValueHolder.addDependent_(this);
		}
	}

	/**
	 * Add an association.
	 *
	 * @param anAssociation StAssociation
	 */
	public void add_(Object anObject) {
		if (anObject instanceof Object[]) {
			Object key = ((Object[]) anObject)[0];
			Object value = ((Object[]) anObject)[1];
			this.soundLevelData().put(key, value);
			this.changed_with_($("wave"), key);
			this.changed_with_($("level"), value);
		}
	}

	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new JunSoundMeterViewAwt(this);
		} else {
			return new JunSoundMeterViewSwing(this);
		}
	}

	/**
	 * Action for the update notification.
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		StSymbol aSymbol = evt.getAspect();
		if (aSymbol == $("value")) {
			if (soundValueHolder != null) {
				this.add_(soundValueHolder.value());
			}
		} else {
			super.update_(evt);
		}
	}
}
