package sra.jun.opengl.display;

import java.awt.*;
import java.util.Vector;
import sra.jun.geometry.basic.*;
import sra.jun.geometry.transformations.*;
import sra.jun.opengl.objects.*;
import sra.jun.opengl.texture.*;
import sra.smalltalk.StBlockClosure;

/**
 * JunOpenGLDisplayModelTestExamples class
 * 
 *  @author    he
 *  @created   1998/11/09 (by he)
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun493 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLDisplayModelTestExamples.java,v 6.6 2004/04/15 06:28:07 nisinaka Exp $
 */
public class JunOpenGLDisplayModelTestExamples extends sra.jun.system.support.JunTestExamples {

	/**
	 * Example1: Shows an axes.
	 * 
	 * @return boolean
	 */
	public static boolean Example1_Axes() {
		JunOpenGL3dObject anAxes = JunOpenGL3dObject.Axes();
		anAxes = anAxes.transform_(Jun3dTransformation.Scale_(2.0));
		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.displayObject_(anAxes);
		displayModel.open();
		return true;
	}

	/**
	 * Example2: Shows a cube.
	 * 
	 * @return boolean
	 */
	public static boolean Example2_Cube() {
		JunOpenGL3dObject aCube = JunOpenGL3dObject.Cube();
		aCube.paint_(new Color(255, 235, 215));
		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.displayObject_(aCube);
		displayModel.open();
		return true;
	}

	/**
	 * Example3: Shows a globe.
	 * 
	 * @return boolean
	 */
	public static boolean Example3_Globe() {
		JunOpenGL3dObject aGlobe = JunOpenGL3dObject.Globe();
		aGlobe.paint_(Color.blue);
		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.displayObject_(aGlobe);
		displayModel.open();
		return true;
	}

	/**
	 * Example8: Shows a benzene.
	 * 
	 * @return boolean
	 */
	public static boolean Example8_Benzene() {
		JunOpenGL3dObject displayObject = JunOpenGL3dObject.Benzene();
		JunOpenGLDisplayModel displayModel = new JunOpenGLDisplayModel();
		displayModel.displayObject_(displayObject);
		displayModel.viewFactor_(1.2);
		displayModel.open();

		return true;
	}

	/**
	 * ExampleD: Shows four kinds of Earth.
	 * 
	 * @return boolean
	 */
	public static boolean ExampleD() {
		StBlockClosure aBlock = new StBlockClosure() {
			public Object value_(Object anImage) {
				Object[] xyPointsAndSphere = JunOpenGL3dObject.XyPointsAndSphere_radius_longitude_latitude_(30, 1, 360, 180);
				Vector points = (Vector) xyPointsAndSphere[0];
				Jun2dPoint[] xyPoints = new Jun2dPoint[points.size()];

				for (int index = 0; index < points.size(); index++) {
					Jun2dPoint aPoint = (Jun2dPoint) points.elementAt(index);
					xyPoints[index] = new Jun2dPoint(aPoint.x(), 1 - aPoint.y());
				}

				JunOpenGL3dCompoundObject anObject = (JunOpenGL3dCompoundObject) xyPointsAndSphere[1];
				JunOpenGLTexture aTexture = JunOpenGLTexture.Image_((Image) anImage);
				aTexture.coordinates_(xyPoints);
				anObject.texture_(aTexture);
				anObject.name_("earth");
				anObject.show();

				return null;
			}
		};

		aBlock.value_(JunOpenGLTextureTestExamples.ImageEarth());
		aBlock.value_(JunOpenGLTextureTestExamples.ImageEarth1());
		aBlock.value_(JunOpenGLTextureTestExamples.ImageEarth2());
		aBlock.value_(JunOpenGLTextureTestExamples.ImageEarth3());

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunOpenGLDisplayModelTestExamples();
	}
}
