package sra.jun.opengl.flux;

import java.awt.Color;
import sra.jun.geometry.basic.Jun3dPoint;
import sra.jun.geometry.basic.JunAngle;
import sra.jun.geometry.curves.Jun3dLine;
import sra.jun.geometry.transformations.Jun3dTransformation;
import sra.jun.opengl.objects.JunOpenGL3dCompoundObject;
import sra.jun.opengl.objects.JunOpenGL3dObject;

/**
 * JunOpenGLFluxModelTestExamples class
 * 
 *  @author    UNKNOWN
 *  @created   UNKNOWN
 *  @updated   N/A
 *  @version   445 (with StPL6.5) based on Jun490 for Smalltalk
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 1999-2004 Information-technology Promotion Agency, Japan (IPA)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: JunOpenGLFluxModelTestExamples.java,v 6.6 2004/04/15 06:28:08 nisinaka Exp $
 */
public class JunOpenGLFluxModelTestExamples extends sra.jun.system.support.JunTestExamples {

	/**
	 * Example: Open a flux model.
	 * 
	 * @return boolean
	 */
	public static boolean Example() {
		JunOpenGLFluxModel model = new JunOpenGLFluxModel();
		model.open();

		return true;
	}

	/**
	 * Example1: 
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		JunOpenGL3dCompoundObject anObject;
		JunOpenGLFluxMutableObject mutableObject;
		JunOpenGLFluxObject fluxObject;
		JunOpenGLFluxModel fluxModel;

		anObject = (JunOpenGL3dCompoundObject) JunOpenGL3dObject.Box();

		for (int i = 0; i < anObject.components().length; i++) {
			anObject.components()[i].paint_(Color.red);
		}

		mutableObject = JunOpenGLFluxMutableObject.Object_(anObject);
		fluxObject = new JunOpenGLFluxObject();
		fluxObject.addMutable_(mutableObject);
		fluxModel = new JunOpenGLFluxModel(fluxObject);
		fluxModel.trackerModel().loop();
		fluxModel.open();

		return true;
	}

	/**
	 * Example3: 
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		JunOpenGL3dObject earthObject;
		JunOpenGL3dObject moonObject;
		JunOpenGLFluxImmutableObject immutableObject;
		JunOpenGLFluxMutableObject mutableObject;
		int angleDegrees;
		JunOpenGLFluxObject fluxObject;
		JunOpenGLFluxModel fluxModel;

		earthObject = JunOpenGL3dObject.Sphere_(20);
		earthObject.paint_(Color.cyan);
		immutableObject = JunOpenGLFluxImmutableObject.Object_(earthObject);
		moonObject = JunOpenGL3dObject.Sphere_(30);
		moonObject = moonObject.scaledBy_(0.3);
		moonObject = moonObject.translatedBy_(new Jun3dPoint(0, 5, 0));
		moonObject.paint_(Color.yellow);
		mutableObject = JunOpenGLFluxMutableObject.Object_(moonObject);
		angleDegrees = 10;

		for (int degrees = 0; degrees < (360 - angleDegrees); degrees += angleDegrees) {
			Jun3dTransformation aT = Jun3dTransformation.RotateZ_(JunAngle.FromDeg_(degrees));
			mutableObject.add_(aT);
		}

		fluxObject = new JunOpenGLFluxObject();
		fluxObject.addImmutable_(immutableObject);
		fluxObject.addMutable_(mutableObject);
		fluxModel = new JunOpenGLFluxModel(fluxObject);
		fluxModel.trackerModel().loop();
		fluxModel.open();

		//fluxModel.start();
		return true;
	}

	/**
	 * Example4: 
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		JunOpenGL3dObject object;
		object = JunOpenGL3dObject.Sphere_(15);
		object.paint_(Color.cyan);

		JunOpenGLFluxImmutableObject earthObject = JunOpenGLFluxImmutableObject.Object_(object);
		object = JunOpenGL3dObject.Sphere_(15);
		object = object.scaledBy_(0.3);
		object = object.translatedBy_(new Jun3dPoint(0, 5, 0));
		object.paint_(Color.yellow);

		JunOpenGLFluxMutableObject firstMoonObject = JunOpenGLFluxMutableObject.Object_(object);
		object = JunOpenGL3dObject.Sphere_(30);
		object = object.scaledBy_(0.3);
		object = object.translatedBy_(new Jun3dPoint(0, -3, 0));
		object = object.transform_(Jun3dTransformation.RotateX_(JunAngle.FromDeg_(30)));
		object.paint_(Color.magenta);

		JunOpenGLFluxMutableObject secondMoonObject = JunOpenGLFluxMutableObject.Object_(object);
		int angleDegrees = 10;

		for (int degrees = 0; degrees < (360 - angleDegrees); degrees += angleDegrees) {
			Jun3dTransformation aT = Jun3dTransformation.RotateZ_(JunAngle.FromDeg_(degrees));
			firstMoonObject.add_(aT);

			Jun3dPoint fromPoint = (Jun3dPoint) earthObject.boundingBox().center();
			Jun3dPoint toPoint = (Jun3dPoint) secondMoonObject.boundingBox().center();
			toPoint = (Jun3dPoint) toPoint.transform_(Jun3dTransformation.RotateX_(JunAngle.FromDeg_(90)));

			Jun3dLine aLine = new Jun3dLine(fromPoint, toPoint);
			aT = Jun3dTransformation.Rotate_around_(JunAngle.FromDeg_(degrees), aLine);
			secondMoonObject.add_(aT);
		}

		JunOpenGLFluxObject fluxObject = new JunOpenGLFluxObject();
		fluxObject.addImmutable_(earthObject);
		fluxObject.addMutable_(firstMoonObject);
		fluxObject.addMutable_(secondMoonObject);

		JunOpenGLFluxModel fluxModel = new JunOpenGLFluxModel(fluxObject);
		fluxModel.trackerModel().loop();
		fluxModel.open();
		//fluxModel.start();

		return true;
	}

	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new JunOpenGLFluxModelTestExamples();
	}
}
