package sra.smalltalk;

import java.awt.*;

/**
 * StView interface
 * 
 *  @author    MATSUDA Ryouichi
 *  @created   1999/06/03 (by MATSUDA Ryouichi)
 *  @updated   1999/11/11 (by MATSUDA Ryouichi)
 *  @updated   2002/01/23 (by nisinaka)
 *  @updated   2002/11/11 (by nisinaka)
 *  @version   6.5
 *  @copyright 1999-2004 SRA (Software Research Associates, Inc.)
 *  @copyright 2001-2004 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * $Id: StView.java,v 6.7 2004/04/15 06:28:22 nisinaka Exp $
 */
public interface StView extends DependentListener {

	/**
	 * Release the receiver.
	 * 
	 * @category initialize-release
	 */
	public void release();

	/**
	 * Answer my model.
	 * 
	 * @return sra.smalltalk.StModel
	 * @category model accessing
	 */
	public StModel model();

	/**
	 * Set my model.
	 * 
	 * @param newModel sra.smalltalk.StModel
	 * @category model accessing
	 */
	public void model_(StModel newModel);

	/**
	 * Answer my controller.
	 * 
	 * @return sra.smalltalk.StController
	 * @category controller accessing
	 */
	public StController controller();

	/**
	 * Set my controller.
	 * 
	 * @param newController sra.smalltalk.StController
	 * @category controller accessing
	 */
	public void controller_(StController newController);

	/**
	 * Answer the view as a java component.
	 *
	 * @return java.awt.Component
	 * @category converting
	 */
	public Component toComponent();

	/**
	 * Answer the top component of the view.
	 *
	 * @return java.awt.Window
	 * @category user interface
	 */
	public Window topComponent();

	/**
	 * Create a Frame for the view.
	 *
	 * @param anApplicationModel sra.smalltalk.StApplicationModel
	 * @return java.awt.Frame
	 * @category interface opening
	 */
	public Frame _createFrame(StApplicationModel anApplicationModel);

	/**
	 * Show the popup menu at the specified point on the view.
	 *
	 * @param x int
	 * @param y int
	 * @category popup menu
	 */
	public void _showPopupMenu(int x, int y);

	/**
	 * Set up the keyboard for the view on the window.
	 * 
	 * @param aWindow java.awt.Window
	 * @category keyboard
	 */
	public void _setupKeyboard(Window aWindow);

}
